##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Contained Tests
"""
from __future__ import absolute_import
import doctest

import unittest

from persistent import Persistent

import zope.interface
import zope.component

from zope.container.contained import ContainedProxy
from zope.container.contained import NameChooser
from zope.container.contained import contained
from zope.container.contained import uncontained
from zope.container.sample import SampleContainer
from zope.container import testing
from zope.container.interfaces import NameReserved, IContainer, IReservedNames

class MyOb(Persistent):
    pass


class TestContainedProxy(unittest.TestCase):

    def test_basic_proxy_attribute_management_and_picklability(self):
        # Contained-object proxy

        # This is a picklable proxy that can be put around objects that
        # don't implement IContained.

        l = [1, 2, 3]
        p = ContainedProxy(l)
        p.__parent__ = 'Dad'
        p.__name__ = 'p'
        self.assertEqual([1, 2, 3], p)
        self.assertEqual('Dad', p.__parent__)
        self.assertEqual('p', p.__name__)

        import pickle
        p2 = pickle.loads(pickle.dumps(p))
        self.assertEqual([1, 2, 3], p2)
        self.assertEqual('Dad', p2.__parent__)
        self.assertEqual('p', p2.__name__)

    def test_declarations_on_ContainedProxy(self):
        # It is possible to make declarations on ContainedProxy objects.
        from zope.container.interfaces import IContained
        from persistent.interfaces import IPersistent

        class I1(zope.interface.Interface):
            pass
        @zope.interface.implementer(I1)
        class C(object):
            pass

        c = C()
        p = ContainedProxy(c)

        # ContainedProxy provides no interfaces on it's own:

        self.assertEqual((),
                         tuple(zope.interface.providedBy(ContainedProxy)))


        # It implements IContained and IPersistent:
        self.assertEqual((IContained, IPersistent),
                         tuple(zope.interface.implementedBy(ContainedProxy)))

        # A proxied object has IContainer, in addition to what the unproxied
        # object has:
        self.assertEqual(tuple(zope.interface.providedBy(p)),
                         (I1, IContained, IPersistent))

        class I2(zope.interface.Interface):
            pass
        zope.interface.directlyProvides(c, I2)
        self.assertEqual(tuple(zope.interface.providedBy(p)),
                         (I2, I1, IContained, IPersistent))

        # We can declare interfaces through the proxy:

        class I3(zope.interface.Interface):
            pass
        zope.interface.directlyProvides(p, I3)
        self.assertEqual(tuple(zope.interface.providedBy(p)),
                         (I3, I1, IContained, IPersistent))


    def test_ContainedProxy_instances_have_no_instance_dictionaries(self):
        # Make sure that proxies don't introduce extra instance dictionaries
        class C(object):
            pass

        c = C()
        c.x = 1
        self.assertEqual(dict(c.__dict__), {'x': 1})

        p = ContainedProxy(c)
        self.assertEqual(dict(p.__dict__), {'x': 1})

        p.y = 3
        self.assertEqual(dict(p.__dict__), {'x': 1, 'y': 3})
        self.assertEqual(dict(c.__dict__), {'x': 1, 'y': 3})

        self.assertIs(p.__dict__, c.__dict__)

    def test_get_set_ProxiedObject(self):
        from zope.container.contained import getProxiedObject
        from zope.container.contained import setProxiedObject

        proxy = ContainedProxy(self)
        self.assertIs(self, getProxiedObject(proxy))

        o = object()
        self.assertIs(o, getProxiedObject(o))

        old = setProxiedObject(proxy, o)
        self.assertIs(self, old)
        self.assertIs(o, getProxiedObject(proxy))

        with self.assertRaises(TypeError):
            setProxiedObject(self, 1)



class TestNameChooser(unittest.TestCase):
    def test_checkName(self):
        container = SampleContainer()
        container['foo'] = 'bar'
        checkName = NameChooser(container).checkName

        # invalid type for the name
        self.assertRaises(TypeError, checkName, 2, object())
        self.assertRaises(TypeError, checkName, [], object())
        self.assertRaises(TypeError, checkName, None, object())
        self.assertRaises(TypeError, checkName, None, None)

        # invalid names
        self.assertRaises(ValueError, checkName, '+foo', object())
        self.assertRaises(ValueError, checkName, '@foo', object())
        self.assertRaises(ValueError, checkName, 'f/oo', object())
        self.assertRaises(ValueError, checkName, '', object())

        # existing names
        self.assertRaises(KeyError, checkName, 'foo', object())
        self.assertRaises(KeyError, checkName, u'foo', object())

        # correct names
        self.assertEqual(True, checkName('2', object()))
        self.assertEqual(True, checkName(u'2', object()))
        self.assertEqual(True, checkName('other', object()))
        self.assertEqual(True, checkName(u'reserved', object()))
        self.assertEqual(True, checkName(u'r\xe9served', object()))

        # reserved names
        @zope.component.adapter(IContainer)
        @zope.interface.implementer(IReservedNames)
        class ReservedNames(object):
            def __init__(self, context):
                self.reservedNames = set(('reserved', 'other'))
        zope.component.getSiteManager().registerAdapter(ReservedNames)

        self.assertRaises(NameReserved, checkName, 'reserved', object())
        self.assertRaises(NameReserved, checkName, 'other', object())
        self.assertRaises(NameReserved, checkName, u'reserved', object())
        self.assertRaises(NameReserved, checkName, u'other', object())

    def test_chooseName(self):
        container = SampleContainer()
        container['foo.old.rst'] = 'rst doc'
        nc = NameChooser(container)

        # correct name without changes
        self.assertEqual(nc.chooseName('foobar.rst', None),
                         u'foobar.rst')
        self.assertEqual(nc.chooseName(u'\xe9', None),
                         u'\xe9')

        # automatically modified named
        self.assertEqual(nc.chooseName('foo.old.rst', None),
                         u'foo.old-2.rst')
        self.assertEqual(nc.chooseName('+@+@foo.old.rst', None),
                         u'foo.old-2.rst')
        self.assertEqual(nc.chooseName('+@+@foo/foo+@', None),
                         u'foo-foo+@')

        # empty name
        self.assertEqual(nc.chooseName('', None), u'NoneType')
        self.assertEqual(nc.chooseName('@+@', []), u'list')

        # if the name is not a string it is converted
        self.assertEqual(nc.chooseName(None, None), u'None')
        self.assertEqual(nc.chooseName(2, None), u'2')
        self.assertEqual(nc.chooseName([], None), u'[]')
        container['None'] = 'something'
        self.assertEqual(nc.chooseName(None, None), u'None-2')
        container['None-2'] = 'something'
        self.assertEqual(nc.chooseName(None, None), u'None-3')

        # even if the given name cannot be converted to unicode
        class BadBoy(object):
            def __unicode__(self):
                raise Exception
            # Py3: Support
            __str__ = __unicode__

        self.assertEqual(nc.chooseName(BadBoy(), set()), u'set')

class TestFunctions(unittest.TestCase):

    def test_contained(self):
        obj = contained(self, 42, 'name')
        self.assertEqual(obj, self)
        self.assertEqual(obj.__name__, 'name')
        self.assertEqual(obj.__parent__, 42)

    def test_uncontained_no_attr(self):
        with self.assertRaises(AttributeError):
            uncontained(self, 42)

    def test_uncontained_no_attr_fixing_up(self):
        # Causes the attribute error to be ignored
        import zope.container.contained
        zope.container.contained.fixing_up = True
        try:
            uncontained(self, 42)
        finally:
            zope.container.contained.fixing_up = False


class TestPlacefulSetup(unittest.TestCase):

    def test_cover(self):
        # We don't actually use this anywhere in this package, this just makes sure
        # it's still around.
        from zope.container.testing import ContainerPlacefulSetup
        setup = ContainerPlacefulSetup()
        setup.setUp()
        setup.buildFolders()
        setup.tearDown()

def test_suite():
    suite = unittest.defaultTestLoader.loadTestsFromName(__name__)
    suite.addTest(doctest.DocTestSuite(
        'zope.container.contained',
        setUp=testing.setUp, tearDown=testing.tearDown,
        checker=testing.checker))
    return suite



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Support for containment constraints

   Either a container or an object can provide constraints on the
   containment relationship.

   A container expresses constraints through a precondition on it's
   `__setitem__` method in it's interface.

   Preconditions can be simple callable objects, like functions. They
   should raise a ``zope.interface.Invalid`` exception to indicate that a
   constraint isn't satisfied:

   >>> def preNoZ(container, name, ob):
   ...     "Silly precondition example"
   ...     if name.startswith("Z"):
   ...         raise zope.interface.Invalid("Names can not start with Z")

   >>> class I1(zope.interface.Interface):
   ...     def __setitem__(name, on):
   ...         "Add an item"
   ...     __setitem__.precondition = preNoZ

   >>> from zope.container.interfaces import IContainer
   >>> @zope.interface.implementer(I1, IContainer)
   ... class C1(object):
   ...     def __repr__(self):
   ...         return 'C1'

   Given such a precondition, we can then check whether an object can be
   added:

   >>> c1 = C1()
   >>> checkObject(c1, "bob", None)
   >>> checkObject(c1, "Zbob", None)
   Traceback (most recent call last):
   ...
   Invalid: Names can not start with Z

   We can also express constaints on the containers an object can be
   added to. We do this by setting a field constraint on an object's
   `__parent__` attribute:

   >>> import zope.schema

   A field constraint is a callable object that returns a boolean value:

   >>> def con1(container):
   ...     "silly container constraint"
   ...     if not hasattr(container, 'x'):
   ...         return False
   ...     return True

   >>> class I2(zope.interface.Interface):
   ...     __parent__ = zope.schema.Field(constraint = con1)

   >>> @zope.interface.implementer(I2)
   ... class O(object):
   ...     pass

   If the constraint isn't satisfied, we'll get a validation error when we
   check whether the object can be added:

   >>> checkObject(c1, "bob", O())
   Traceback (most recent call last):
   ...
   ConstraintNotSatisfied: (C1, '__parent__')

   Note that the validation error isn't very informative. For that
   reason, it's better for constraints to raise Invalid errors when they
   aren't satisfied:

   >>> def con1(container):
   ...     "silly container constraint"
   ...     if not hasattr(container, 'x'):
   ...         raise zope.interface.Invalid("What, no x?")
   ...     return True

   >>> class I2(zope.interface.Interface):
   ...     __parent__ = zope.schema.Field(constraint = con1)

   >>> @zope.interface.implementer(I2)
   ... class O(object):
   ...     pass

   >>> checkObject(c1, "bob", O())
   Traceback (most recent call last):
   ...
   Invalid: What, no x?

   >>> c1.x = 1
   >>> checkObject(c1, "bob", O())

   The `checkObject` function is handy when checking whether we can add an
   existing object to a container, but, sometimes, we want to check
   whether an object produced by a factory can be added.  To do this, we
   use `checkFactory`:

   >>> class Factory(object):
   ...     def __call__(self):
   ...         return O()
   ...     def getInterfaces(self):
   ...         return zope.interface.implementedBy(O)

   >>> factory = Factory()

   >>> checkFactory(c1, "bob", factory)
   True

   >>> del c1.x
   >>> checkFactory(c1, "bob", factory)
   False

   Unlike `checkObject`, `checkFactory`:

   - Returns a boolean value

   - Takes a factory (e.g. a class) rather than an argument.

   The container constraint we defined for C1 isn't actually used to
   check the factory:

   >>> c1.x = 1
   >>> checkFactory(c1, "Zbob", factory)
   True

   To work with `checkFactory`, a container precondition has to
   implement a factory method.  This is because a factory, rather than
   an object is passed.  To illustrate this, we'll make preNoZ its own
   factory method:

   >>> preNoZ.factory = preNoZ

   We can do this (silly thing) because preNoZ doesn't use the object
   argument.

   >>> checkFactory(c1, "Zbob", factory)
   False
   """
__docformat__ = 'restructuredtext'

import sys

from zope.dottedname.resolve import resolve
import zope.schema
from zope.interface import providedBy
from zope.container.interfaces import InvalidItemType, InvalidContainerType
from zope.container.i18n import ZopeMessageFactory as _
from zope.container.interfaces import IContainer

def checkObject(container, name, object):
    """Check containment constraints for an object and container
    """

    # check __setitem__ precondition
    containerProvided = providedBy(container)
    __setitem__ = containerProvided.get('__setitem__')
    if __setitem__ is not None:
        precondition = __setitem__.queryTaggedValue('precondition')
        if precondition is not None:
            precondition(container, name, object)

    # check that object is not being pasted into itself or its children.
    target = container
    while target is not None:
        if target is object:
            raise TypeError("Cannot add an object to itself or its children.")
        if zope.location.interfaces.ILocation.providedBy(target):
            target = target.__parent__
        else:
            target = None

    # check the constraint on __parent__
    __parent__ = providedBy(object).get('__parent__')
    try:
        validate = __parent__.validate
    except AttributeError:
        pass
    else:
        validate(container)


    if not containerProvided.extends(IContainer):
        # If it doesn't implement IContainer, it can't contain stuff.
        raise TypeError(
            _('Container is not a valid Zope container.')
            )

def checkFactory(container, name, factory):
    __setitem__ = providedBy(container).get('__setitem__')

    try:
        precondition = __setitem__.queryTaggedValue('precondition')
        precondition = precondition.factory
    except AttributeError:
        pass
    else:
        try:
            precondition(container, name, factory)
        except zope.interface.Invalid:
            return False

    # check the constraint on __parent__
    __parent__ = factory.getInterfaces().get('__parent__')
    try:
        validate = __parent__.validate
    except AttributeError:
        pass
    else:
        try:
            validate(container)
        except zope.interface.Invalid:
            return False

    return True


class readproperty(object):

    def __init__(self, func):
        self.func = func

    def __get__(self, inst, class_):
        if inst is None:
            return self

        func = self.func
        return func(inst)


class IItemTypePrecondition(zope.interface.Interface):

    def __call__(container, name, object):
        """Test whether container setitem arguments are valid.

        Raise zope.interface.Invalid if the object is invalid.
        """

    def factory(container, name, factory):
        """Test whether objects provided by the factory are acceptable

        Return a boolean value.
        """


class _TypesBased(object):

    @readproperty
    def types(self):
        raw_types, module = self.raw_types
        types = []
        for t in raw_types:
            if isinstance(t, str):
                t = resolve(t, module)
            types.append(t)

        self.types = types
        return types

    def __init__(self, *types, **kw):
        if [t for t in types if isinstance(t, str)]:
            # have dotted names
            module = kw.get('module', sys._getframe(1).f_globals['__name__'])
            self.raw_types = types, module
        else:
            self.types = types


@zope.interface.implementer(IItemTypePrecondition)
class ItemTypePrecondition(_TypesBased):
    """Specify a `__setitem__` precondition that restricts item types

    Items must be one of the given types.

    >>> class I1(zope.interface.Interface):
    ...     pass
    >>> class I2(zope.interface.Interface):
    ...     pass


    >>> precondition = ItemTypePrecondition(I1, I2)

    >>> class Ob(object):
    ...     pass
    >>> ob = Ob()

    >>> class Factory(object):
    ...     def __call__(self):
    ...         return Ob()
    ...     def getInterfaces(self):
    ...         return zope.interface.implementedBy(Ob)

    >>> factory = Factory()

    >>> try:
    ...     precondition(None, 'foo', ob)
    ... except InvalidItemType as v:
    ...     v.args[0], (v.args[1] is ob), (v.args[2] == (I1, I2))
    ... else:
    ...     print('Should have failed')
    (None, True, True)

    >>> try:
    ...     precondition.factory(None, 'foo', factory)
    ... except InvalidItemType as v:
    ...     v.args[0], (v.args[1] is factory), (v.args[2] == (I1, I2))
    ... else:
    ...     print('Should have failed')
    (None, True, True)

    >>> zope.interface.classImplements(Ob, I2)
    >>> precondition(None, 'foo', ob)
    >>> precondition.factory(None, 'foo', factory)

    """


    def __call__(self, container, name, object):
        for iface in self.types:
            if iface.providedBy(object):
                return
        raise InvalidItemType(container, object, self.types)

    def factory(self, container, name, factory):
        implemented = factory.getInterfaces()

        for iface in self.types:
            if implemented.isOrExtends(iface):
                return
        raise InvalidItemType(container, factory, self.types)


def contains(*types):
    """Declare that a container type contains only the given types

    This is used within a class suite defining an interface to create
    a __setitem__ specification with a precondition allowing only the
    given types:

      >>> class IFoo(zope.interface.Interface):
      ...     pass
      >>> class IBar(zope.interface.Interface):
      ...     pass
      >>> class IFooBarContainer(IContainer):
      ...     contains(IFoo, IBar)

      >>> __setitem__ = IFooBarContainer['__setitem__']
      >>> __setitem__.getTaggedValue('precondition').types == (IFoo, IBar)
      True

    It is invalid to call contains outside a class suite:

      >>> contains(IFoo, IBar)
      Traceback (most recent call last):
      ...
      TypeError: contains not called from suite
    """

    frame = sys._getframe(1)
    f_locals = frame.f_locals
    f_globals = frame.f_globals

    if not (f_locals is not f_globals
            and f_locals.get('__module__')
            and f_locals.get('__module__') == f_globals.get('__name__')
    ):
        raise TypeError("contains not called from suite")

    def __setitem__(key, value):
        """
        This serves as a copy of IContainer.__setitem__ to hold
        the ``precondition`` attribute. Note that it replaces a local
        __setitem__ defined before.
        """
    __setitem__.__doc__ = IContainer['__setitem__'].__doc__
    __setitem__.precondition = ItemTypePrecondition(
        *types,
        **dict(module=f_globals['__name__'])
        )
    f_locals['__setitem__'] = __setitem__


class IContainerTypesConstraint(zope.interface.Interface):

    def __call__(object):
        """Test whether object is valid.

        Return True if valid.
        Raise zope.interface.Invalid if the objet is invalid.
        """


@zope.interface.implementer(IContainerTypesConstraint)
class ContainerTypesConstraint(_TypesBased):
    """Constrain a container to be one of a number of types

    >>> class I1(zope.interface.Interface):
    ...     pass
    >>> class I2(zope.interface.Interface):
    ...     pass
    >>> class Ob(object):
    ...     pass
    >>> ob = Ob()
    >>> constraint = ContainerTypesConstraint(I1, I2)
    >>> try:
    ...     constraint(ob)
    ... except InvalidContainerType as v:
    ...     (v.args[0] is ob), (v.args[1] == (I1, I2))
    ... else:
    ...     print('Should have failed')
    (True, True)

    >>> zope.interface.classImplements(Ob, I2)
    >>> constraint(Ob())
    True
    """
    def __call__(self, object):
        for iface in self.types:
            if iface.providedBy(object):
                return True
        raise InvalidContainerType(object, self.types)


def containers(*types):
    """Declare the container types a type can be contained in

    This is used within a class suite defining an interface to create
    a __parent__ specification with a constraint allowing only the
    given types:

      >>> class IFoo(IContainer):
      ...     pass
      >>> class IBar(IContainer):
      ...     pass

      >>> from zope.location.interfaces import IContained
      >>> class IFooBarContained(IContained):
      ...     containers(IFoo, IBar)

      >>> __parent__ = IFooBarContained['__parent__']
      >>> __parent__.constraint.types == (IFoo, IBar)
      True

    It is invalid to call containers outside a class suite:

      >>> containers(IFoo, IBar)
      Traceback (most recent call last):
      ...
      TypeError: containers not called from suite
    """

    frame = sys._getframe(1)
    f_locals = frame.f_locals
    f_globals = frame.f_globals

    if not (f_locals is not f_globals
            and f_locals.get('__module__')
            and f_locals.get('__module__') == f_globals.get('__name__')
    ):
        raise TypeError("containers not called from suite")

    __parent__ = zope.schema.Field(
        constraint=ContainerTypesConstraint(
            *types,
            **dict(module=f_globals['__name__'])
        )
    )
    f_locals['__parent__'] = __parent__



from zope.interface import implementer
from zope.location.interfaces import IContained

from zope.container.interfaces import IContainer
from zope.container.constraints import containers
from zope.container.constraints import contains

class IBuddyFolder(IContainer):
    contains('.IBuddy')

class IBuddy(IContained):
    containers(IBuddyFolder)

@implementer(IBuddy)
class Buddy:
    pass

@implementer(IBuddyFolder)
class BuddyFolder:
    pass

class IContact(IContained):
    containers('.IContacts')

class IContacts(IContainer):
    contains(IContact)

@implementer(IContact)
class Contact:
    pass

@implementer(IContacts)
class Contacts:
    pass



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Container Traverser tests.
"""
from __future__ import absolute_import
import unittest
from zope.testing.cleanup import CleanUp
from zope.interface import implementer
from zope.traversing.interfaces import TraversalError

from zope.container.traversal import ContainerTraversable
from zope.container.interfaces import IContainer
import six

@implementer(IContainer)
class Container(object):

    def __init__(self, attrs=None, objs=None):
        for attr, value in six.iteritems(attrs or {}):
            setattr(self, attr, value)

        self.__objs = {}
        for name, value in six.iteritems(objs or {}):
            self.__objs[name] = value

    def get(self, name, default=None):
        return self.__objs.get(name, default)


class Test(CleanUp, unittest.TestCase):
    def testAttr(self):
        # test container path traversal
        foo = Container()
        bar = Container()
        baz = Container()
        c = Container({'foo': foo}, {'bar': bar, 'foo': baz})

        T = ContainerTraversable(c)
        self.assertTrue(T.traverse('foo', []) is baz)
        self.assertTrue(T.traverse('bar', []) is bar)
        self.assertRaises(TraversalError, T.traverse, 'morebar', [])

    def test_unicode_obj(self):
        # test traversal with unicode
        voila = Container()
        c = Container({}, {u'voil\xe0': voila})
        self.assertIs(ContainerTraversable(c).traverse(u'voil\xe0', []),
                      voila)

    def test_unicode_attr(self):
        c = Container()
        with self.assertRaises(TraversalError):
            ContainerTraversable(c).traverse(u'voil\xe0', [])


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

from zope.proxy import AbstractPyProxyBase

_MARKER = object()

from persistent import Persistent

def _special_name(name):
    "attribute names we delegate to Persistent for"
    return (name.startswith('_Persistent')
            or name.startswith('_p_')
            or name.startswith('_v_')
            or name in PyContainedProxyBase.__slots__)

class PyContainedProxyBase(AbstractPyProxyBase, Persistent):
    """Persistent proxy
    """
    __slots__ = ('_wrapped', '__parent__', '__name__', '__weakref__')

    def __new__(cls, obj):
        inst = super(PyContainedProxyBase, cls).__new__(cls, obj)
        inst.__parent__ = None
        inst.__name__ = None
        return inst

    def __init__(self, obj):
        super(PyContainedProxyBase, self).__init__(obj)
        self.__parent__ = None
        self.__name__ = None

    def __reduce__(self):
        return (type(self),
                (self._wrapped,),
                (self.__parent__, self.__name__))

    def __reduce_ex__(self, protocol):
        return self.__reduce__()

    def __setstate__(self, state):
        object.__setattr__(self, '__parent__', state[0])
        object.__setattr__(self, '__name__', state[1])

    def __getstate__(self):
        return (self.__parent__, self.__name__)

    def __getnewargs__(self):
        return self._wrapped,

    def _p_invalidate(self):
        # The superclass wants to clear the __dict__, which
        # we don't have, but we would otherwise delegate
        # to the wrapped object, which is clearly wrong in this case.
        # This method is a copy of the super method with
        # clearing the dict omitted
        if self._Persistent__jar is not None:
            if self._Persistent__flags is not None:
                self._Persistent__flags = None
            try:
                object.__getattribute__(self, '__dict__').clear()
            except AttributeError:
                pass

    # Attribute protocol
    def __getattribute__(self, name):
        if _special_name(name):
            # Our own attribute names need to go to Persistent so we get
            # activated
            return Persistent.__getattribute__(self, name)

        if name in ('__reduce__', '__reduce_ex__', '__getstate__', '__setstate__', '__getnewargs__'):
            return object.__getattribute__(self, name)

        return super(PyContainedProxyBase,self).__getattribute__(name)

    def __setattr__(self, name, value):
        if _special_name(name):
            # Our own attribute names need to go directly to Persistent
            # so that _p_changed gets set, in addition to the _p values themselves
            return Persistent.__setattr__(self, name, value)

        return super(PyContainedProxyBase, self).__setattr__(name, value)


def py_getProxiedObject(obj):
    if isinstance(obj, PyContainedProxyBase):
        return obj._wrapped
    return obj


def py_setProxiedObject(obj, new_value):
    if not isinstance(obj, PyContainedProxyBase):
        raise TypeError('Not a proxy')
    old, obj._wrapped = obj._wrapped, new_value
    return old



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Container-related interfaces
"""
__docformat__ = 'restructuredtext'

from zope.interface import Interface, Invalid
from zope.interface.common.mapping import IItemMapping
from zope.interface.common.mapping import IReadMapping, IEnumerableMapping
from zope.location.interfaces import ILocation
from zope.schema import Set

from zope.lifecycleevent.interfaces import IObjectModifiedEvent

# the following imports provide backwards compatibility for consumers;
# do not remove them
from zope.lifecycleevent.interfaces import IObjectMovedEvent
from zope.lifecycleevent.interfaces import IObjectAddedEvent
from zope.lifecycleevent.interfaces import IObjectRemovedEvent
from zope.location.interfaces import IContained
# /end backwards compatibility imports

from zope.container.i18n import ZopeMessageFactory as _

class DuplicateIDError(KeyError):
    pass

class ContainerError(Exception):
    """An error of a container with one of its components."""

class InvalidContainerType(Invalid, TypeError):
    """The type of a container is not valid."""

class InvalidItemType(Invalid, TypeError):
    """The type of an item is not valid."""

class InvalidType(Invalid, TypeError):
    """The type of an object is not valid."""



class IItemContainer(IItemMapping):
    """Minimal readable container."""


class ISimpleReadContainer(IItemContainer, IReadMapping):
    """Readable content containers."""


class IReadContainer(ISimpleReadContainer, IEnumerableMapping):
    """Readable containers that can be enumerated."""


class IWriteContainer(Interface):
    """An interface for the write aspects of a container."""

    def __setitem__(name, object):
        """Add the given `object` to the container under the given name.

        Raises a ``TypeError`` if the key is not a unicode or ascii string.

        Raises a ``ValueError`` if the key is empty, or if the key contains
        a character which is not allowed in an object name.

        Raises a ``KeyError`` if the key violates a uniqueness constraint.

        The container might choose to add a different object than the
        one passed to this method.

        If the object doesn't implement `IContained`, then one of two
        things must be done:

        1. If the object implements `ILocation`, then the `IContained`
           interface must be declared for the object.

        2. Otherwise, a `ContainedProxy` is created for the object and
           stored.

        The object's `__parent__` and `__name__` attributes are set to the
        container and the given name.

        If the old parent was ``None``, then an `IObjectAddedEvent` is
        generated, otherwise, an `IObjectMovedEvent` is generated.  An
        `IContainerModifiedEvent` is generated for the container.

        If the object replaces another object, then the old object is
        deleted before the new object is added, unless the container
        vetos the replacement by raising an exception.

        If the object's `__parent__` and `__name__` were already set to
        the container and the name, then no events are generated and
        no hooks.  This allows advanced clients to take over event
        generation.

        """

    def __delitem__(name):
        """Delete the named object from the container.

        Raises a ``KeyError`` if the object is not found.

        If the deleted object's `__parent__` and `__name__` match the
        container and given name, then an `IObjectRemovedEvent` is
        generated and the attributes are set to ``None``. If the object
        can be adapted to `IObjectMovedEvent`, then the adapter's
        `moveNotify` method is called with the event.

        Unless the object's `__parent__` and `__name__` attributes were
        initially ``None``, generate an `IContainerModifiedEvent` for the
        container.

        If the object's `__parent__` and `__name__` were already set to
        ``None``, then no events are generated.  This allows advanced
        clients to take over event generation.

        """


class IItemWriteContainer(IWriteContainer, IItemContainer):
    """A write container that also supports minimal reads."""


class IContainer(IReadContainer, IWriteContainer):
    """Readable and writable content container."""


class IContentContainer(IContainer):
    """A container that is to be used as a content type."""


class IBTreeContainer(IContainer):
    """Container that supports BTree semantics for some methods."""

    def items(key=None):
        """Return an iterator over the key-value pairs in the container.

        If ``None`` is passed as `key`, this method behaves as if no argument
        were passed; exactly as required for ``IContainer.items()``.

        If `key` is in the container, the first item provided by the iterator
        will correspond to that key.  Otherwise, the first item will be for
        the key that would come next if `key` were in the container.

        """

    def keys(key=None):
        """Return an iterator over the keys in the container.

        If ``None`` is passed as `key`, this method behaves as if no argument
        were passed; exactly as required for ``IContainer.keys()``.

        If `key` is in the container, the first key provided by the iterator
        will be that key.  Otherwise, the first key will be the one that would
        come next if `key` were in the container.

        """

    def values(key=None):
        """Return an iterator over the values in the container.

        If ``None`` is passed as `key`, this method behaves as if no argument
        were passed; exactly as required for ``IContainer.values()``.

        If `key` is in the container, the first value provided by the iterator
        will correspond to that key.  Otherwise, the first value will be for
        the key that would come next if `key` were in the container.

        """


class IOrdered(Interface):
    """Objects whose contents are maintained in order."""


    def updateOrder(order):
        """Revise the order of keys, replacing the current ordering.

        order is a list or a tuple containing the set of existing keys in
        the new order. `order` must contain ``len(keys())`` items and cannot
        contain duplicate keys.

        Raises ``TypeError`` if order is not a tuple or a list.

        Raises ``ValueError`` if order contains an invalid set of keys.
        """


class IOrderedContainer(IOrdered, IContainer):
    """Containers whose contents are maintained in order."""



class IContainerNamesContainer(IContainer):
    """Containers that always choose names for their items."""

class IReservedNames(Interface):
    """A sequence of names that are reserved for that container"""
    
    reservedNames = Set(
        title=_(u'Reserved Names'),
        description=_(u'Names that are not allowed for addable content'),
        required=True,
        )
    
class NameReserved(ValueError):
    __doc__ = _("""The name is reserved for this container""")

##############################################################################
# Adding objects

class UnaddableError(ContainerError):
    """An object cannot be added to a container."""

    def __init__(self, container, obj, message=""):
        self.container = container
        self.obj = obj
        self.message = message and ": %s" % message

    def __str__(self):
        return ("%(obj)s cannot be added "
                "to %(container)s%(message)s" % self.__dict__)


class INameChooser(Interface):

    def checkName(name, object):
        """Check whether an object name is valid.

        Raises a user error if the name is not valid.
        """

    def chooseName(name, object):
        """Choose a unique valid name for the object.

        The given name and object may be taken into account when
        choosing the name.

        chooseName is expected to always choose a valid name (that would pass
        the checkName test) and never raise an error.

        """


##############################################################################
# Modifying containers


class IContainerModifiedEvent(IObjectModifiedEvent):
    """The container has been modified.

    This event is specific to "containerness" modifications, which means
    addition, removal or reordering of sub-objects.
    """


##############################################################################
# Finding objects

class IFind(Interface):
    """
    Find support for containers.
    """

    def find(id_filters=None, object_filters=None):
        """Find object that matches all filters in all sub-objects.

        This container itself is not included.
        """


class IObjectFindFilter(Interface):

    def matches(object):
        """Return True if the object matches the filter criteria."""


class IIdFindFilter(Interface):

    def matches(id):
        """Return True if the id matches the filter criteria."""



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""FS-based directory implementation tests for containers
"""

import doctest
import unittest

from zope.container import testing
from zope.container import directory


class Directory(object):
    a = None

class TestCloner(unittest.TestCase):

    def test_Cloner(self):
        d = Directory()
        d.a = 1
        clone = directory.Cloner(d)('foo')
        self.assertIsNot(clone, d)
        self.assertEqual(clone.__class__, d.__class__)


class TestNoop(unittest.TestCase):

    def test_noop(self):
        self.assertIs(self, directory.noop(self))


class TestRootDirectoryFactory(unittest.TestCase):

    def test_returns_folder(self):
        from zope.container.folder import Folder
        factory = directory.RootDirectoryFactory(None)
        self.assertIsInstance(factory(None), Folder)

def test_suite():
    flags = doctest.ELLIPSIS|doctest.NORMALIZE_WHITESPACE
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocFileSuite("directory.rst",
                             setUp=testing.setUp, tearDown=testing.tearDown,
                             optionflags=flags),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test the OrderedContainer.
"""
import unittest
from doctest import DocTestSuite

from zope.component.eventtesting import getEvents, clearEvents
from zope.container import testing
from zope.container.tests.test_icontainer import TestSampleContainer

class TestOrderedContainer(TestSampleContainer):

    def test_order_events(self):
        # Prepare the setup::
        from zope.container.contained import ContainerModifiedEvent

        # Prepare some objects::

        from zope.container.ordered import OrderedContainer
        oc = OrderedContainer()
        oc['foo'] = 'bar'
        oc['baz'] = 'quux'
        oc['zork'] = 'grue'
        self.assertEqual(oc.keys(),
                         ['foo', 'baz', 'zork'])

        # Now change the order::

        clearEvents()
        oc.updateOrder(['baz', 'foo', 'zork'])
        self.assertEqual(oc.keys(),
                         ['baz', 'foo', 'zork'])

        # Check what events have been sent::

        events = getEvents()
        self.assertEqual(1, len(events))
        self.assertIsInstance(events[0], ContainerModifiedEvent)

        # This is in fact a specialized modification event::

        from zope.lifecycleevent.interfaces import IObjectModifiedEvent
        self.assertTrue(IObjectModifiedEvent.providedBy(events[0]))


    def test_order_all_items_available_at_object_added_event(self):
        # Now register an event subscriber to object added events.
        import zope.component
        from zope.lifecycleevent.interfaces import IObjectAddedEvent

        keys = []
        @zope.component.adapter(IObjectAddedEvent)
        def printContainerKeys(event):
            keys.extend(event.newParent.keys())


        zope.component.provideHandler(printContainerKeys)

        # Now we are adding an object to the container.

        from zope.container.ordered import OrderedContainer
        oc = OrderedContainer()
        oc['foo'] = 'FOO'

        self.assertEqual(keys, ['foo'])

    def test_order_adding_none(self):
        # This is a regression test: adding None to an OrderedContainer
        # used to corrupt its internal data structure (_order and _data
        # would get out of sync, causing KeyErrors when you tried to iterate).

        from zope.container.ordered import OrderedContainer
        oc = OrderedContainer()
        oc['foo'] = None
        self.assertEqual(oc.keys(), ['foo'])
        self.assertEqual(oc.values(), [None])
        self.assertEqual(oc.items(),
                         [('foo', None)])
        # None got proxied, so 'is None is not true
        self.assertIsNotNone(oc['foo'])
        self.assertEqual(None, oc['foo'])

    def test_order_updateOrder_bytes(self):
        # https://github.com/zopefoundation/zope.container/issues/21
        from zope.container.ordered import OrderedContainer
        keys = [u'a', u'b']
        oc = OrderedContainer()
        oc[keys[0]] = 0
        oc[keys[1]] = 1

        self.assertEqual(keys, oc.keys())

        # Updating with bytes keys...
        oc.updateOrder((b'b', b'a'))
        # still produces text keys
        text_type = str if str is not bytes else unicode
        self.assertEqual(list(reversed(keys)), oc.keys())
        self.assertIsInstance(oc.keys()[0], text_type)


def test_suite():
    suite = unittest.TestSuite([
        unittest.defaultTestLoader.loadTestsFromName(__name__),
    ])
    suite.addTest(DocTestSuite("zope.container.ordered",
                               setUp=testing.setUp,
                               tearDown=testing.tearDown,
                               checker=testing.checker))

    return suite

if __name__ == '__main__':
    unittest.main()



##############################################################################
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################
"""File-system representation adapters for containers

This module includes two adapters (adapter factories, really) for
providing a file-system representation for containers:

`noop`
  Factory that "adapts" `IContainer` to `IWriteDirectory`.
  This is a lie, since it just returns the original object.

`Cloner`
  An `IDirectoryFactory` adapter that just clones the original object.
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer

from zope.component.interfaces import ISite
from zope.container.folder import Folder
from zope.security.proxy import removeSecurityProxy
import zope.filerepresentation.interfaces

from six.moves import map


MARKER = object()

def noop(container):
    """Adapt an `IContainer` to an `IWriteDirectory` by just returning it

    This "works" because `IContainer` and `IWriteDirectory` have the same
    methods, however, the output doesn't actually implement `IWriteDirectory`.
    """
    return container


@implementer(zope.filerepresentation.interfaces.IDirectoryFactory)
class Cloner(object):
    """
    `IContainer` to
    :class:`zope.filerepresentation.interfaces.IDirectoryFactory` adapter
    that clones.

    This adapter provides a factory that creates a new empty container
    of the same class as it's context.
    """
    def __init__(self, context):
        self.context = context

    def __call__(self, name):

        # We remove the security proxy so we can actually call the
        # class and return an unproxied new object.  (We can't use a
        # trusted adapter, because the result must be unproxied.)  By
        # registering this adapter, one effectively gives permission
        # to clone the class.  Don't use this for classes that have
        # exciting side effects as a result of instantiation. :)

        return removeSecurityProxy(self.context).__class__()


class RootDirectoryFactory(object):

    def __init__(self, context):
        pass

    def __call__(self, name):
        return Folder()


class ReadDirectory(object):
    """Adapter to provide a file-system rendition of folders."""

    def __init__(self, context):
        self.context = context

    def keys(self):
        keys = self.context.keys()
        if ISite.providedBy(self.context):
            return list(keys) + ['++etc++site']
        return keys

    def get(self, key, default=None):
        if key == '++etc++site' and ISite.providedBy(self.context):
            return self.context.getSiteManager()
        return self.context.get(key, default)

    def __iter__(self):
        return iter(self.keys())

    def __getitem__(self, key):
        v = self.get(key, MARKER)
        if v is MARKER:
            raise KeyError(key)
        return v

    def values(self):
        return map(self.get, self.keys())

    def __len__(self):
        l = len(self.context)
        if ISite.providedBy(self.context):
            l += 1
        return l

    def items(self):
        get = self.get
        return [(key, get(key)) for key in self.keys()]

    def __contains__(self, key):
        return self.get(key) is not None



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Unit test logic for setting up and tearing down basic infrastructure
"""
import re

import zope.interface
import zope.traversing.testing
from zope import component
from zope.component.testing import PlacelessSetup as CAPlacelessSetup
from zope.component.eventtesting import PlacelessSetup as EventPlacelessSetup
from zope.traversing.interfaces import ITraversable, IContainmentRoot
from zope.testing import renormalizing

from zope.container.interfaces import IWriteContainer, INameChooser
from zope.container.contained import NameChooser
from zope.container.interfaces import ISimpleReadContainer
from zope.container.traversal import ContainerTraversable
from zope.container.sample import SampleContainer

checker = renormalizing.RENormalizing([
    # Python 3 unicode removed the "u".
    (re.compile("u('.*?')"),
     r"\1"),
    (re.compile('u(".*?")'),
     r"\1"),
    # Python 3 renamed type to class.
    (re.compile('<type '),
     r"<class "),
    # Python 3 adds module name to exceptions.
    (re.compile("zope.interface.exceptions.Invalid"),
     r"Invalid"),
    (re.compile("zope.container.interfaces.InvalidContainerType"),
     r"InvalidContainerType"),
    (re.compile("zope.container.interfaces.InvalidItemType"),
     r"InvalidItemType"),
    (re.compile("zope.container.interfaces.NameReserved"),
     r"NameReserved"),
    (re.compile("zope.schema._bootstrapinterfaces.ConstraintNotSatisfied"),
     r"ConstraintNotSatisfied"),
    ])


# XXX we would like to swap the names of the *PlacelessSetup classes
# in here as that would seem to follow the convention better, but
# unfortunately that would break compatibility with zope.app.testing
# (which expects this PlacelessSetup) so it will have to wait.

class PlacelessSetup(object):

    def setUp(self):
        component.provideAdapter(NameChooser, (IWriteContainer,), INameChooser)

class ContainerPlacelessSetup(CAPlacelessSetup,
                              EventPlacelessSetup,
                              PlacelessSetup):

    def setUp(self, doctesttest=None):
        CAPlacelessSetup.setUp(self)
        EventPlacelessSetup.setUp(self)
        PlacelessSetup.setUp(self)

ps = ContainerPlacelessSetup()
setUp = ps.setUp

def tearDown():
    tearDown_ = ps.tearDown
    def tearDown(doctesttest=None):
        tearDown_()
    return tearDown

tearDown = tearDown()

del ps

class ContainerPlacefulSetup(ContainerPlacelessSetup):
    def setUp(self, doctesttest=None):
        ContainerPlacelessSetup.setUp(self, doctesttest)
        zope.traversing.testing.setUp()
        component.provideAdapter(ContainerTraversable,
                                 (ISimpleReadContainer,), ITraversable)

    def tearDown(self, docttesttest=None):
        ContainerPlacelessSetup.tearDown(self)


    def buildFolders(self):
        root = self.rootFolder = SampleContainer()
        zope.interface.directlyProvides(root, IContainmentRoot)
        root[u'folder1'] = SampleContainer()
        root[u'folder1'][u'folder1_1'] = SampleContainer()
        root[u'folder1'][u'folder1_1'][u'folder1_1_1'] = SampleContainer()
        root[u'folder2'] = SampleContainer()
        root[u'folder2'][u'folder2_1'] = SampleContainer()
        root[u'folder2'][u'folder2_1'][u'folder2_1_1'] = SampleContainer()



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""BTree Container Tests
"""
import unittest

from zope.interface.verify import verifyObject

from zope.container.tests.test_icontainer import TestSampleContainer
from zope.container.btree import BTreeContainer
from zope.container.interfaces import IBTreeContainer
from zope.container.interfaces import IContainerModifiedEvent
from zope.container.contained import Contained
from zope.lifecycleevent.interfaces import IObjectRemovedEvent


class TestBTreeContainer(TestSampleContainer, unittest.TestCase):

    def makeTestObject(self):
        return BTreeContainer()


class TestBTreeSpecials(unittest.TestCase):

    def testStoredLength(self):
        # This is a lazy property for backward compatibility.  If the len is not
        # stored already we set it to the length of the underlying
        # btree.
        lazy = BTreeContainer._BTreeContainer__len
        self.assertIs(lazy, BTreeContainer.__dict__['_BTreeContainer__len'])
        self.assertTrue(hasattr(lazy, '__get__'))

        bc = BTreeContainer()
        self.assertEqual(bc.__dict__['_BTreeContainer__len'](), 0)
        del bc.__dict__['_BTreeContainer__len']
        self.assertFalse('_BTreeContainer__len' in bc.__dict__)
        bc['1'] = 1
        self.assertEqual(len(bc), 1)
        self.assertEqual(bc.__dict__['_BTreeContainer__len'](), 1)

        del bc.__dict__['_BTreeContainer__len']
        self.assertFalse('_BTreeContainer__len' in bc.__dict__)
        self.assertEqual(len(bc), 1)
        self.assertEqual(bc.__dict__['_BTreeContainer__len'](), 1)


    # The tests which follow test the additional signatures and declarations
    # for the BTreeContainer that allow it to provide the IBTreeContainer
    # interface.

    def testBTreeContainerInterface(self):
        bc = BTreeContainer()
        self.assertTrue(verifyObject(IBTreeContainer, bc))
        self.checkIterable(bc.items())
        self.checkIterable(bc.keys())
        self.checkIterable(bc.values())

    def testEmptyItemsWithArg(self):
        bc = BTreeContainer()
        self.assertEqual(list(bc.items(None)), list(bc.items()))
        self.assertEqual(list(bc.items("")), [])
        self.assertEqual(list(bc.items("not-there")), [])
        self.checkIterable(bc.items(None))
        self.checkIterable(bc.items(""))
        self.checkIterable(bc.items("not-there"))

    def testEmptyKeysWithArg(self):
        bc = BTreeContainer()
        self.assertEqual(list(bc.keys(None)), list(bc.keys()))
        self.assertEqual(list(bc.keys("")), [])
        self.assertEqual(list(bc.keys("not-there")), [])
        self.checkIterable(bc.keys(None))
        self.checkIterable(bc.keys(""))
        self.checkIterable(bc.keys("not-there"))

    def testEmptyValuesWithArg(self):
        bc = BTreeContainer()
        self.assertEqual(list(bc.values(None)), list(bc.values()))
        self.assertEqual(list(bc.values("")), [])
        self.assertEqual(list(bc.values("not-there")), [])
        self.checkIterable(bc.values(None))
        self.checkIterable(bc.values(""))
        self.checkIterable(bc.values("not-there"))

    def testNonemptyItemsWithArg(self):
        bc = BTreeContainer()
        bc["0"] = 1
        bc["1"] = 2
        bc["2"] = 3
        self.assertEqual(list(bc.items(None)), list(bc.items()))
        self.assertEqual(list(bc.items("")), [("0", 1), ("1", 2), ("2", 3)])
        self.assertEqual(list(bc.items("3")), [])
        self.assertEqual(list(bc.items("2.")), [])
        self.assertEqual(list(bc.items("2")), [("2", 3)])
        self.assertEqual(list(bc.items("1.")), [("2", 3)])
        self.assertEqual(list(bc.items("1")), [("1", 2), ("2", 3)])
        self.assertEqual(list(bc.items("0.")), [("1", 2), ("2", 3)])
        self.assertEqual(list(bc.items("0")), [("0", 1), ("1", 2), ("2", 3)])
        self.checkIterable(bc.items(None))
        self.checkIterable(bc.items(""))
        self.checkIterable(bc.items("0."))
        self.checkIterable(bc.items("3"))

    def testNonemptyKeysWithArg(self):
        bc = BTreeContainer()
        bc["0"] = 1
        bc["1"] = 2
        bc["2"] = 3
        self.assertEqual(list(bc.keys(None)), list(bc.keys()))
        self.assertEqual(list(bc.keys("")), ["0", "1", "2"])
        self.assertEqual(list(bc.keys("3")), [])
        self.assertEqual(list(bc.keys("2.")), [])
        self.assertEqual(list(bc.keys("2")), ["2"])
        self.assertEqual(list(bc.keys("1.")), ["2"])
        self.assertEqual(list(bc.keys("1")), ["1", "2"])
        self.assertEqual(list(bc.keys("0.")), ["1", "2"])
        self.assertEqual(list(bc.keys("0")), ["0", "1", "2"])
        self.checkIterable(bc.keys(None))
        self.checkIterable(bc.keys(""))
        self.checkIterable(bc.keys("0."))
        self.checkIterable(bc.keys("3"))

    def testNonemptyValueWithArg(self):
        bc = BTreeContainer()
        bc["0"] = 1
        bc["1"] = 2
        bc["2"] = 3
        self.assertEqual(list(bc.values(None)), list(bc.values()))
        self.assertEqual(list(bc.values("")), [1, 2, 3])
        self.assertEqual(list(bc.values("3")), [])
        self.assertEqual(list(bc.values("2.")), [])
        self.assertEqual(list(bc.values("2")), [3])
        self.assertEqual(list(bc.values("1.")), [3])
        self.assertEqual(list(bc.values("1")), [2, 3])
        self.assertEqual(list(bc.values("0.")), [2, 3])
        self.assertEqual(list(bc.values("0")), [1, 2, 3])
        self.checkIterable(bc.values(None))
        self.checkIterable(bc.values(""))
        self.checkIterable(bc.values("0."))
        self.checkIterable(bc.values("3"))

    def testCorrectLengthWhenAddingExistingItem(self):
        """
        for bug #175388
        """
        bc = BTreeContainer()
        bc[u'x'] = object()
        self.assertEqual(len(bc), 1)
        bc[u'x'] = bc[u'x']
        self.assertEqual(len(bc), 1)
        self.assertEqual(list(bc), [u'x'])


    def checkIterable(self, iterable):
        it = iter(iterable)
        self.assertTrue(callable(it.__iter__))
        self.assertTrue(iter(it) is it)
        # Exhaust the iterator:
        first_time = list(it)
        self.assertRaises(StopIteration, next, it)
        # Subsequent iterations will return the same values:
        self.assertEqual(list(iterable), first_time)
        self.assertEqual(list(iterable), first_time)


class TestBTreeEvents(unittest.TestCase):

    def setUp(self):
        from zope.event import subscribers
        self._old_subscribers = subscribers[:]
        subscribers[:] = []

    def tearDown(self):
        from zope.event import subscribers
        subscribers[:] = self._old_subscribers

    def testDeletion(self):
        from zope.event import subscribers
        tree = BTreeContainer()
        item = Contained()
        tree['42'] = item
        events = []
        def subscriber(event):
            events.append(event)
            # events should happen after the deletion, not before)
            self.assertEqual(len(tree), 0)
            self.assertEqual(list(tree), [])
        subscribers.append(subscriber)

        del tree['42']
        self.assertEqual(item.__name__, None)
        self.assertEqual(item.__parent__, None)

        self.assertEqual(len(events), 2)
        self.assertTrue(IObjectRemovedEvent.providedBy(events[0]))
        self.assertTrue(IContainerModifiedEvent.providedBy(events[1]))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')




##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Adapters that give the size of an object.
"""
__docformat__ = 'restructuredtext'

from zope.container.i18n import ZopeMessageFactory as _
from zope.size.interfaces import ISized
from zope.interface import implementer

@implementer(ISized)
class ContainerSized(object):
    """
    Implements :class:`zope.size.interfaces.ISize` for
    :class:`zope.container.interfaces.IReadContainer`
    """

    def __init__(self, container):
        self._container = container

    def sizeForSorting(self):
        """See `ISized`"""
        return ('item', len(self._container))

    def sizeForDisplay(self):
        """See `ISized`"""
        num_items = len(self._container)
        if num_items == 1:
            return _('1 item')
        return _('${items} items', mapping={'items': str(num_items)})



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Traversal components for containers
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer, providedBy
from zope.component import queryMultiAdapter, getSiteManager
from zope.component import ComponentLookupError
from zope.traversing.interfaces import TraversalError, ITraversable
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.xmlrpc import IXMLRPCPublisher
from zope.publisher.interfaces import IDefaultViewName, NotFound

from zope.container.interfaces import ISimpleReadContainer, IItemContainer
from zope.container.interfaces import IReadContainer
from six.moves import map


# Note that the next two classes are included here because they
# can be used for multiple view types.

@implementer(IBrowserPublisher, IXMLRPCPublisher)
class ContainerTraverser(object):
    """A traverser that knows how to look up objects by name in a container."""

    __used_for__ = ISimpleReadContainer

    def __init__(self, container, request):
        self.context = container
        self.request = request

    def publishTraverse(self, request, name):
        """See zope.publisher.interfaces.IPublishTraverse"""
        subob = self.context.get(name, None)
        if subob is None:
            view = queryMultiAdapter((self.context, request), name=name)
            if view is not None:
                return view

            raise NotFound(self.context, name, request)

        return subob

    def browserDefault(self, request):
        """See zope.publisher.browser.interfaces.IBrowserPublisher"""
        # XXX this re-implements zope.app.publisher.browser.getDefaultViewName()
        # to break our only dependency on it.
        view_name = getSiteManager(None).adapters.lookup(
            map(providedBy, (self.context, request)), IDefaultViewName)
        if view_name is None:
            raise ComponentLookupError("Couldn't find default view name",
                                       self.context, request)
        view_uri = "@@%s" %view_name
        return self.context, (view_uri,)


class ItemTraverser(ContainerTraverser):
    """A traverser that knows how to look up objects by name in an item
    container."""

    __used_for__ = IItemContainer

    def publishTraverse(self, request, name):
        """See zope.publisher.interfaces.IPublishTraverse"""
        try:
            return self.context[name]
        except KeyError:
            view = queryMultiAdapter((self.context, request), name=name)
            if view is not None:
                return view

        raise NotFound(self.context, name, request)


_marker = object()

@implementer(ITraversable)
class ContainerTraversable(object):
    """Traverses containers via `getattr` and `get`."""

    __used_for__ = IReadContainer

    def __init__(self, container):
        self._container = container


    def traverse(self, name, furtherPath):
        container = self._container

        v = container.get(name, _marker)
        if v is _marker:
            try:
                # Note that if name is a unicode string, getattr will
                # implicitly try to encode it using the system
                # encoding (usually ascii). Failure to encode means
                # invalid attribute name.
                v = getattr(container, name, _marker)
            except UnicodeEncodeError:
                raise TraversalError(container, name)
            if v is _marker:
                raise TraversalError(container, name)

        return v



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Container Traverser Tests
"""

import unittest
from zope.interface import Interface, implementer
from zope import component
from zope.publisher.interfaces import NotFound, IDefaultViewName
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from zope.container.traversal import ContainerTraverser
from zope.container.traversal import ItemTraverser
from zope.container.interfaces import IReadContainer
from zope.container import testing

@implementer(IReadContainer)
class TestContainer(object):

    def __init__(self, **kw):
        for name, value in kw.items():
            setattr(self, name, value)

    def get(self, name, default=None):
        return getattr(self, name, default)

    def __getitem__(self, name):
        v = object()
        o = self.get(name, v)
        if o is v:
            raise KeyError(name)
        return o

class View(object):
    def __init__(self, context, request):
        self.context = context
        self.request = request


class TraverserTest(testing.ContainerPlacelessSetup, unittest.TestCase):

    # The following two methods exist, so that other container traversers can
    # use these tests as a base.
    def _getTraverser(self, context, request):
        return ContainerTraverser(context, request)

    def _getContainer(self, **kw):
        return TestContainer(**kw)

    def setUp(self):
        super(TraverserTest, self).setUp()
        # Create a small object tree
        self.container = self._getContainer()
        self.subcontainer = self._getContainer(Foo=self.container)
        # Initiate a request
        self.request = TestRequest()
        # Create the traverser
        self.traverser = self._getTraverser(self.subcontainer, self.request)
        # Define a simple view for the container
        component.provideAdapter(
            View, (IReadContainer, IDefaultBrowserLayer), Interface,
            name='viewfoo')

    def test_itemTraversal(self):
        self.assertEqual(
            self.traverser.publishTraverse(self.request, 'Foo'),
            self.container)
        self.assertRaises(
            NotFound,
            self.traverser.publishTraverse, self.request, 'morebar')

    def test_viewTraversal(self):
        self.assertEqual(
            self.traverser.publishTraverse(self.request, 'viewfoo').__class__,
            View)
        self.assertEqual(
            self.traverser.publishTraverse(self.request, 'Foo'),
            self.container)
        self.assertRaises(
            NotFound,
            self.traverser.publishTraverse, self.request, 'morebar')
        self.assertRaises(
            NotFound,
            self.traverser.publishTraverse, self.request, '@@morebar')

    def test_browserDefault_without_registration_should_raise(self):
        self.assertRaises(component.ComponentLookupError,
                          self.traverser.browserDefault, self.request)

    def test_browserDefault(self):
        component.provideAdapter(
            'myDefaultView', (Interface, IDefaultBrowserLayer),
            IDefaultViewName)
        self.assertEqual((self.subcontainer, ('@@myDefaultView',)),
                         self.traverser.browserDefault(self.request))


class TestItemTraverser(TraverserTest):

    def _getTraverser(self, context, request):
        return ItemTraverser(context, request)

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""The standard Zope Folder.
"""
__docformat__ = 'restructuredtext'
from zope.interface import implementer
from zope.container import btree, interfaces

@implementer(interfaces.IContentContainer)
class Folder(btree.BTreeContainer):
    """The standard Zope Folder implementation."""

    # BBB: The data attribute used to be exposed.
    # For compatibility with existing pickles, we read and write that name
    @property
    def data(self):
        return self._SampleContainer__data

    @data.setter
    def data(self, value):
        self._SampleContainer__data = value

    def __getstate__(self):
        state = super(Folder, self).__getstate__()
        data = state.pop('_SampleContainer__data')
        state['data'] = data
        return state

    def __setstate__(self, state):
        if 'data' in state and '_SampleContainer__data' not in state:
            state['_SampleContainer__data'] = state.pop('data')
        super(Folder, self).__setstate__(state)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test container ISized adapter.
"""
import unittest

from zope.interface import implementer
from zope.size.interfaces import ISized
from zope.container.interfaces import IContainer

@implementer(IContainer)
class DummyContainer(object):

    def __init__(self, numitems):
        self._numitems = numitems

    def __len__(self):
        return self._numitems


class Test(unittest.TestCase):

    def testImplementsISized(self):
        from zope.container.size import ContainerSized
        sized = ContainerSized(DummyContainer(23))
        self.assertTrue(ISized.providedBy(sized))

    def testEmptyContainer(self):
        from zope.container.size import ContainerSized
        obj = DummyContainer(0)
        sized = ContainerSized(obj)
        self.assertEqual(sized.sizeForSorting(), ('item', 0))
        self.assertEqual(sized.sizeForDisplay(), u'${items} items')
        self.assertEqual(sized.sizeForDisplay().mapping['items'], '0')

    def testOneItem(self):
        from zope.container.size import ContainerSized
        obj = DummyContainer(1)
        sized = ContainerSized(obj)
        self.assertEqual(sized.sizeForSorting(), ('item', 1))
        self.assertEqual(sized.sizeForDisplay(), u'1 item')

    def testSeveralItems(self):
        from zope.container.size import ContainerSized
        obj = DummyContainer(2)
        sized = ContainerSized(obj)
        self.assertEqual(sized.sizeForSorting(), ('item', 2))
        self.assertEqual(sized.sizeForDisplay(), u'${items} items')
        self.assertEqual(sized.sizeForDisplay().mapping['items'], '2')

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Find functionality tests
"""
import unittest

from zope.container.interfaces import IReadContainer
from zope.container.interfaces import IObjectFindFilter
from zope.container.find import FindAdapter, SimpleIdFindFilter
from zope.container.find import SimpleInterfacesFindFilter
from zope.interface import implementer, Interface, directlyProvides

@implementer(IReadContainer)
class FakeContainer(object):

    def __init__(self, id, objects):
        self._id = id
        self._objects = objects

    def items(self):
        return [(object._id, object) for object in self._objects]

    def __len__(self):
        return len(self._objects)

class FakeInterfaceFoo(Interface):
    """Test interface Foo"""

class FakeInterfaceBar(Interface):
    """Test interface Bar"""

class FakeInterfaceSpam(Interface):
    """Test interface Spam"""

@implementer(IObjectFindFilter)
class TestObjectFindFilter(object):

    def __init__(self, count):
        self._count = count

    def matches(self, object):
        if not IReadContainer.providedBy(object):
            raise AssertionError("Expecting container")
        return len(object) == self._count


class Test(unittest.TestCase):
    def test_idFind(self):
        alpha = FakeContainer('alpha', [])
        delta = FakeContainer('delta', [])
        beta = FakeContainer('beta', [delta])
        gamma = FakeContainer('gamma', [])
        tree = FakeContainer(
            'tree',
            [alpha, beta, gamma])
        find = FindAdapter(tree)
        # some simple searches
        result = find.find([SimpleIdFindFilter(['beta'])])
        self.assertEqual([beta], result)
        result = find.find([SimpleIdFindFilter(['gamma'])])
        self.assertEqual([gamma], result)
        result = find.find([SimpleIdFindFilter(['delta'])])
        self.assertEqual([delta], result)
        # we should not find the container we search on
        result = find.find([SimpleIdFindFilter(['tree'])])
        self.assertEqual([], result)
        # search for multiple ids
        result = find.find([SimpleIdFindFilter(['alpha', 'beta'])])
        self.assertEqual([alpha, beta], result)
        result = find.find([SimpleIdFindFilter(['beta', 'delta'])])
        self.assertEqual([beta, delta], result)
        # search without any filters, find everything
        result = find.find([])
        self.assertEqual([alpha, beta, delta, gamma], result)
        # search for something that doesn't exist
        result = find.find([SimpleIdFindFilter(['foo'])])
        self.assertEqual([], result)
        # find for something that has two ids at the same time,
        # can't ever be the case
        result = find.find([SimpleIdFindFilter(['alpha']),
                            SimpleIdFindFilter(['beta'])])
        self.assertEqual([], result)

    def test_objectFind(self):
        alpha = FakeContainer('alpha', [])
        delta = FakeContainer('delta', [])
        beta = FakeContainer('beta', [delta])
        gamma = FakeContainer('gamma', [])
        tree = FakeContainer(
            'tree',
            [alpha, beta, gamma])
        find = FindAdapter(tree)
        result = find.find(object_filters=[TestObjectFindFilter(0)])
        self.assertEqual([alpha, delta, gamma], result)
        result = find.find(object_filters=[TestObjectFindFilter(1)])
        self.assertEqual([beta], result)
        result = find.find(object_filters=[TestObjectFindFilter(2)])
        self.assertEqual([], result)

    def test_combinedFind(self):
        alpha = FakeContainer('alpha', [])
        delta = FakeContainer('delta', [])
        beta = FakeContainer('beta', [delta])
        gamma = FakeContainer('gamma', [])
        tree = FakeContainer(
            'tree',
            [alpha, beta, gamma])
        find = FindAdapter(tree)
        result = find.find(id_filters=[SimpleIdFindFilter(['alpha'])],
                           object_filters=[TestObjectFindFilter(0)])
        self.assertEqual([alpha], result)

        result = find.find(id_filters=[SimpleIdFindFilter(['alpha'])],
                           object_filters=[TestObjectFindFilter(1)])
        self.assertEqual([], result)

    def test_interfaceFind(self):
        alpha = FakeContainer('alpha', [])
        directlyProvides(alpha, FakeInterfaceBar)
        delta = FakeContainer('delta', [])
        directlyProvides(delta, FakeInterfaceFoo)
        beta = FakeContainer('beta', [delta])
        directlyProvides(beta, FakeInterfaceSpam)
        gamma = FakeContainer('gamma', [])
        tree = FakeContainer(
            'tree',
            [alpha, beta, gamma])
        find = FindAdapter(tree)
        result = find.find(object_filters=[
            SimpleInterfacesFindFilter(FakeInterfaceFoo, FakeInterfaceSpam)])
        self.assertEqual([beta, delta], result)

    def test_find_non_container_return_default(self):
        data = {'a': 42}
        self.assertEqual(FindAdapter(data).find(), [42])

def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Find Support
"""
__docformat__ = 'restructuredtext'

from zope.interface import implementer
from .interfaces import IFind, IIdFindFilter, IObjectFindFilter
from .interfaces import IReadContainer

@implementer(IFind)
class FindAdapter(object):
    """Adapts :class:`zope.container.interfaces.IReadContainer`"""

    __used_for__ = IReadContainer

    def __init__(self, context):
        self._context = context

    def find(self, id_filters=None, object_filters=None):
        'See IFind'
        id_filters = id_filters or []
        object_filters = object_filters or []
        result = []
        container = self._context
        for id, object in container.items():
            _find_helper(id, object, container,
                         id_filters, object_filters,
                         result)
        return result


def _find_helper(id, object, container, id_filters, object_filters, result):
    for id_filter in id_filters:
        if not id_filter.matches(id):
            break
    else:
        # if we didn't break out of the loop, all name filters matched
        # now check all object filters
        for object_filter in object_filters:
            if not object_filter.matches(object):
                break
        else:
            # if we didn't break out of the loop, all filters matched
            result.append(object)

    if not IReadContainer.providedBy(object):
        return

    container = object
    for id, object in container.items():
        _find_helper(id, object, container, id_filters, object_filters, result)

@implementer(IIdFindFilter)
class SimpleIdFindFilter(object):
    """Filter objects by ID"""

    def __init__(self, ids):
        self._ids = ids

    def matches(self, id):
        'See INameFindFilter'
        return id in self._ids

@implementer(IObjectFindFilter)
class SimpleInterfacesFindFilter(object):
    """Filter objects on the provided interfaces"""

    def __init__(self, *interfaces):
        self.interfaces = interfaces

    def matches(self, object):
        for iface in self.interfaces:
            if iface.providedBy(object):
                return True
        return False



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Container constraint tests
"""
import unittest

class TestUnaddableError(unittest.TestCase):

    def test_str(self):
        from zope.container.interfaces import UnaddableError
        e = UnaddableError("'container'", "'obj'", "Extra")
        self.assertEqual(str(e),
                         "'obj' cannot be added to 'container': Extra")

class TestCheckObject(unittest.TestCase):

    def test_non_container(self):
        from zope.container.constraints import checkObject

        with self.assertRaisesRegexp(TypeError,
                                     "Container is not a valid Zope container."):
            checkObject({}, 'foo', 42)


class TestCheckFactory(unittest.TestCase):

    def test_no_validate_on_parent(self):
        # Does nothing if no constraints are provided
        from zope.container.constraints import checkFactory

        class Factory(object):
            def getInterfaces(self):
                return {}

        result = checkFactory({}, 'name', Factory())
        self.assertTrue(result)


class TestTypesBased(unittest.TestCase):

    def test_raw_types(self):
        from zope.container.constraints import _TypesBased

        t = _TypesBased('.TestTypesBased', module=__name__)
        self.assertEqual(t.types, [TestTypesBased])

    def test_raw_types_property(self):
        from zope.container.constraints import _TypesBased

        t = _TypesBased.types
        self.assertTrue(hasattr(t, '__get__'))

def test_suite():
    import doctest
    from zope.container import testing
    return unittest.TestSuite((
        unittest.defaultTestLoader.loadTestsFromName(__name__),
        doctest.DocTestSuite(
            'zope.container.constraints',
            checker=testing.checker),
        doctest.DocFileSuite(
            '../constraints.rst',
            checker=testing.checker),
    ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# http://docs.zope.org/zopetoolkit
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.container package
"""
import os
import platform

from setuptools import setup, find_packages, Extension

def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()

def alltests():
    import sys
    import unittest
    # use the zope.testrunner machinery to find all the
    # test suites we've put under ourselves
    import zope.testrunner.find
    import zope.testrunner.options
    here = os.path.abspath(os.path.join(os.path.dirname(__file__), 'src'))
    args = sys.argv[:]
    defaults = ["--test-path", here]
    options = zope.testrunner.options.get_options(args, defaults)
    suites = list(zope.testrunner.find.find_suites(options))
    return unittest.TestSuite(suites)

# PyPy cannot correctly build the C optimizations, and even if it
# could they would be anti-optimizations (the C extension
# compatibility layer is known-slow, and defeats JIT opportunities).
py_impl = getattr(platform, 'python_implementation', lambda: None)
pure_python = os.environ.get('PURE_PYTHON', False)
is_pypy = py_impl() == 'PyPy'

if pure_python or is_pypy:
    ext_modules = []
else:
    ext_modules = [Extension("zope.container._zope_container_contained",
                             [os.path.join("src", "zope", "container",
                                           "_zope_container_contained.c")
                             ], include_dirs=['include']),
    ]

install_requires = [
    'setuptools',
    'six',
    'zope.interface',
    'zope.dottedname',
    'zope.schema',
    'zope.component',
    'zope.event',
    'zope.location>=3.5.4',
    'zope.security',
    'zope.lifecycleevent>=3.5.2',
    'zope.i18nmessageid',
    'zope.filerepresentation',
    'zope.size',
    'zope.traversing>=4.0.0a1',
    'zope.publisher',
    'zope.proxy>=4.1.5',
    'persistent>=4.1.0',
    'BTrees'
]


setup(name='zope.container',
      version=read('version.txt').strip(),
      author='Zope Foundation and Contributors',
      author_email='zope-dev@zope.org',
      description='Zope Container',
      long_description=(
          read('README.rst')
          + '\n\n' +
          read('CHANGES.rst')
      ),
      keywords="zope container",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope3',
      ],
      url='http://github.com/zopefoundation/zope.container',
      license='ZPL 2.1',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['zope'],
      ext_modules=ext_modules,
      extras_require={
          'docs': [
              'Sphinx',
              'repoze.sphinx.autointerface',
          ],
          'test': [
              'zope.testing',
              'zope.testrunner',
          ],
          'zcml': [
              'zope.component[zcml]',
              'zope.configuration',
              'zope.security[zcml]>=4.0.0a3',
          ],
          'zodb': [
              'ZODB>=3.10',
          ],
      },
      install_requires=install_requires,
      tests_require=[
          'zope.testing',
          'zope.testrunner',
      ],
      test_suite='__main__.alltests',
      include_package_data=True,
      zip_safe=False,
)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Ordered container implementation.
"""
__docformat__ = 'restructuredtext'

from persistent import Persistent
from persistent.dict import PersistentDict
from persistent.list import PersistentList
from zope.container.interfaces import IOrderedContainer
from zope.interface import implementer
from zope.container.contained import Contained, setitem, uncontained
from zope.container.contained import notifyContainerModified
from zope.container.contained import checkAndConvertName

@implementer(IOrderedContainer)
class OrderedContainer(Persistent, Contained):
    """ `OrderedContainer` maintains entries' order as added and moved.

    >>> oc = OrderedContainer()
    >>> int(IOrderedContainer.providedBy(oc))
    1
    >>> len(oc)
    0
    """
    def __init__(self):

        self._data = PersistentDict()
        self._order = PersistentList()

    def keys(self):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc.keys()
        ['foo']
        >>> oc['baz'] = 'quux'
        >>> oc.keys()
        ['foo', 'baz']
        >>> int(len(oc._order) == len(oc._data))
        1
        """

        return self._order[:]

    def __iter__(self):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc['baz'] = 'quux'
        >>> [i for i in oc]
        ['foo', 'baz']
        >>> int(len(oc._order) == len(oc._data))
        1
        """

        return iter(self.keys())

    def __getitem__(self, key):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc['foo'] = 'bar'
        >>> oc['foo']
        'bar'
        """

        return self._data[key]

    def get(self, key, default=None):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc['foo'] = 'bar'
        >>> oc.get('foo')
        'bar'
        >>> oc.get('funky', 'No chance, dude.')
        'No chance, dude.'
        """

        return self._data.get(key, default)

    def values(self):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc.values()
        ['bar']
        >>> oc['baz'] = 'quux'
        >>> oc.values()
        ['bar', 'quux']
        >>> int(len(oc._order) == len(oc._data))
        1
        """

        return [self._data[i] for i in self._order]

    def __len__(self):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> int(len(oc) == 0)
        1
        >>> oc['foo'] = 'bar'
        >>> int(len(oc) == 1)
        1
        """

        return len(self._data)

    def items(self):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc.items()
        [('foo', 'bar')]
        >>> oc['baz'] = 'quux'
        >>> oc.items()
        [('foo', 'bar'), ('baz', 'quux')]
        >>> int(len(oc._order) == len(oc._data))
        1
        """

        return [(i, self._data[i]) for i in self._order]

    def __contains__(self, key):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc['foo'] = 'bar'
        >>> int('foo' in oc)
        1
        >>> int('quux' in oc)
        0
        """

        return key in self._data

    has_key = __contains__

    def _setitemf(self, key, value):
        if key not in self._data:
            self._order.append(key)
        self._data[key] = value

    def __setitem__(self, key, object):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc._order
        ['foo']
        >>> oc['baz'] = 'quux'
        >>> oc._order
        ['foo', 'baz']
        >>> int(len(oc._order) == len(oc._data))
        1

        >>> oc['foo'] = 'baz'
        Traceback (most recent call last):
        ...
        KeyError: u'foo'
        >>> oc._order
        ['foo', 'baz']
        """
        # This function creates a lot of events that other code
        # listens to. None of them are fired (notified) though, before
        # _setitemf is called. At that point we need to be sure that
        # we have the key in _order too.
        setitem(self, self._setitemf, key, object)
        return key

    def __delitem__(self, key):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc.keys()
        []
        >>> oc['foo'] = 'bar'
        >>> oc['baz'] = 'quux'
        >>> oc['zork'] = 'grue'
        >>> oc.items()
        [('foo', 'bar'), ('baz', 'quux'), ('zork', 'grue')]
        >>> int(len(oc._order) == len(oc._data))
        1
        >>> del oc['baz']
        >>> oc.items()
        [('foo', 'bar'), ('zork', 'grue')]
        >>> int(len(oc._order) == len(oc._data))
        1
        """

        uncontained(self._data[key], self, key)
        del self._data[key]
        self._order.remove(key)

    def updateOrder(self, order):
        """ See `IOrderedContainer`.

        >>> oc = OrderedContainer()
        >>> oc['foo'] = 'bar'
        >>> oc['baz'] = 'quux'
        >>> oc['zork'] = 'grue'
        >>> oc.keys()
        ['foo', 'baz', 'zork']
        >>> oc.updateOrder(['baz', 'foo', 'zork'])
        >>> oc.keys()
        ['baz', 'foo', 'zork']
        >>> oc.updateOrder(['baz', 'zork', 'foo'])
        >>> oc.keys()
        ['baz', 'zork', 'foo']
        >>> oc.updateOrder(['baz', 'zork', 'foo'])
        >>> oc.keys()
        ['baz', 'zork', 'foo']
        >>> oc.updateOrder(('zork', 'foo', 'baz'))
        >>> oc.keys()
        ['zork', 'foo', 'baz']
        >>> oc.updateOrder(['baz', 'zork'])
        Traceback (most recent call last):
        ...
        ValueError: Incompatible key set.
        >>> oc.updateOrder(['foo', 'bar', 'baz', 'quux'])
        Traceback (most recent call last):
        ...
        ValueError: Incompatible key set.
        >>> oc.updateOrder(1)
        Traceback (most recent call last):
        ...
        TypeError: order must be a tuple or a list.
        >>> oc.updateOrder('bar')
        Traceback (most recent call last):
        ...
        TypeError: order must be a tuple or a list.
        >>> oc.updateOrder(['baz', 'zork', 'quux'])
        Traceback (most recent call last):
        ...
        ValueError: Incompatible key set.
        >>> del oc['baz']
        >>> del oc['zork']
        >>> del oc['foo']
        >>> len(oc)
        0
        """

        if not isinstance(order, (list, tuple)):
            raise TypeError('order must be a tuple or a list.')

        if len(order) != len(self._order):
            raise ValueError("Incompatible key set.")

        order = [checkAndConvertName(x) for x in order]

        if frozenset(order) != frozenset(self._order):
            raise ValueError("Incompatible key set.")

        if order == self._order:
            return

        self._order[:] = order

        notifyContainerModified(self)



##############################################################################
#
# Copyright (c) 2013 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Contained Tests
"""

import gc
import unittest

try:
    from ZODB.DemoStorage import DemoStorage
    from ZODB.DB import DB
    import transaction
    HAVE_ZODB = True
except ImportError: # pragma: no cover
    HAVE_ZODB = False

from persistent import Persistent

from zope.container.contained import ContainedProxy


class MyOb(Persistent):
    pass

@unittest.skipUnless(HAVE_ZODB, "Needs ZODB")
class TestContainedZODB(unittest.TestCase):

    def setUp(self):
        self.db = DB(DemoStorage('test_storage'))

    def tearDown(self):
        self.db.close()

    def test_basic_persistent_w_non_persistent_proxied(self):
        p = ContainedProxy([1])
        p.__parent__ = 2
        p.__name__ = 'test'
        db = self.db
        c = db.open()
        c.root()['p'] = p
        transaction.commit()

        c2 = db.open()
        p2 = c2.root()['p']
        self.assertEqual([1], p2)
        self.assertEqual(2, p2.__parent__)
        self.assertEqual('test', p2.__name__)

        self.assertEqual(0, p2._p_changed)
        p2._p_deactivate()
        self.assertFalse(bool(p2._p_changed))

        self.assertEqual(p2.__name__, 'test')

    def test_basic_persistent_w_persistent_proxied(self):
        # Here, we'll verify that shared references work and
        # that updates to both the proxies and the proxied objects
        # are made correctly.
        #
        #        ----------------------
        #        |                    |
        #      parent                other
        #        |                 /
        #       ob  <--------------

        # Here we have an object, parent, that contains ob.  There is another
        # object, other, that has a non-container reference to ob.

        parent = MyOb()
        parent.ob = ContainedProxy(MyOb())
        parent.ob.__parent__ = parent
        parent.ob.__name__ = 'test'
        other = MyOb()
        other.ob = parent.ob

        # We can change ob through either parent or other

        parent.ob.x = 1
        other.ob.y = 2

        # Now we'll save the data:

        db = self.db
        c1 = db.open()
        c1.root()['parent'] = parent
        c1.root()['other'] = other
        transaction.commit()

        # We'll open a second connection and verify that we have the data we
        # expect:

        c2 = db.open()
        p2 = c2.root()['parent']
        self.assertIs(p2.ob.__parent__, p2)

        self.assertEqual(1, p2.ob.x)
        self.assertEqual(2, p2.ob.y)

        o2 = c2.root()['other']
        self.assertIs(o2.ob, p2.ob)
        self.assertIs(o2.ob, p2.ob)
        self.assertEqual('test', o2.ob.__name__)


        # Now we'll change things around a bit. We'll move things around
        # a bit. We'll also add an attribute to ob

        o2.ob.__name__ = 'test 2'
        o2.ob.__parent__ = o2
        o2.ob.z = 3

        self.assertIsNot(p2.ob.__parent__, p2)
        self.assertIs(p2.ob.__parent__, o2)

        # And save the changes:

        transaction.commit()

        # Now we'll reopen the first connection and verify that we can see
        # the changes:

        c1.close()
        c1 = db.open()
        p2 = c1.root()['parent']
        self.assertEqual('test 2', p2.ob.__name__)

        self.assertEqual(3, p2.ob.z)
        self.assertIs(p2.ob.__parent__, c1.root()['other'])

    def test_proxy_cache_interaction(self):
        # Test to make sure the proxy properly interacts with the object cache

        # Persistent objects are their own weak refs.  Thier deallocators
        # need to notify their connection's cache that their object is being
        # deallocated, so that it is removed from the cache.

        db = self.db
        db.setCacheSize(5)
        conn = db.open()
        conn.root()['p'] = ContainedProxy(None)

        # We need to create some filler objects to push our proxy out of the cache:

        for i in range(10):
            conn.root()[i] = MyOb()

        transaction.commit()

        # Let's get the oid of our proxy:

        oid = conn.root()['p']._p_oid

        # Now, we'll access the filler object's:
        for i in range(10):
            getattr(conn.root()[i], 'x', 0)

        # We've also accessed the root object. If we garbage-collect the
        # cache:

        conn._cache.incrgc()

        # Then the root object will still be active, because it was accessed
        # recently:

        self.assertEqual(0, conn.root()._p_changed)

        # And the proxy will be in the cache, because it's refernced from
        # the root object:

        self.assertIsNotNone(conn._cache.get(oid))

        # But it's a ghost:

        self.assertFalse(bool(conn.root()['p']._p_changed))


        # If we deactivate the root object:

        conn.root()._p_deactivate()

        # Then we'll release the last reference to the proxy and it should
        # no longer be in the cache. To be sure, we'll call gc:

        gc.collect()
        self.assertIsNone(conn._cache.get(oid))


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)


if __name__ == '__main__':
    unittest.main()



from unittest import TestCase, makeSuite

from zope.container.folder import Folder
from zope.container.tests.test_icontainer import TestSampleContainer

import pickle

class Test(TestSampleContainer, TestCase):

    def makeTestObject(self):
        return Folder()

    def testDataAccess(self):
        folder = self.makeTestObject()
        self.assertNotEqual(folder.data, None)
        folder.data = 'foo'
        self.assertEqual(folder.data, 'foo')

    def testPickleCompatibility(self):
        folder = self.makeTestObject()
        folder['a'] = 1
        self.assertEqual(folder.__getstate__()['data'], folder._SampleContainer__data)

        folder_clone = pickle.loads(pickle.dumps(folder))
        self.assertEqual(folder_clone['a'], folder['a'])


def test_suite():
    return makeSuite(Test)



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Sample container implementation.

This is primarily for testing purposes.

It might be useful as a mix-in for some classes, but many classes will
need a very different implementation.
"""
__docformat__ = 'restructuredtext'

from zope.container.interfaces import IContainer
from zope.interface import implementer
from zope.container.contained import Contained, setitem, uncontained


@implementer(IContainer)
class SampleContainer(Contained):
    """Sample container implementation suitable for testing.

    It is not suitable, directly as a base class unless the subclass
    overrides `_newContainerData` to return a persistent mapping object.
    """

    def __init__(self):
        self.__data = self._newContainerData()

    def _newContainerData(self):
        """Construct an item-data container

        Subclasses should override this if they want different data.

        The value returned is a mapping object that also has `get`,
        `has_key`, `keys`, `items`, and `values` methods.
        """
        return {}

    def keys(self):
        '''See interface `IReadContainer`'''
        return self.__data.keys()

    def __iter__(self):
        return iter(self.__data)

    def __getitem__(self, key):
        '''See interface `IReadContainer`'''
        return self.__data[key]

    def get(self, key, default=None):
        '''See interface `IReadContainer`'''
        return self.__data.get(key, default)

    def values(self):
        '''See interface `IReadContainer`'''
        return self.__data.values()

    def __len__(self):
        '''See interface `IReadContainer`'''
        return len(self.__data)

    def items(self):
        '''See interface `IReadContainer`'''
        return self.__data.items()

    def __contains__(self, key):
        '''See interface `IReadContainer`'''
        return key in self.__data

    has_key = __contains__

    def __setitem__(self, key, object):
        '''See interface `IWriteContainer`'''
        setitem(self, self.__data.__setitem__, key, object)

    def __delitem__(self, key):
        '''See interface `IWriteContainer`'''
        uncontained(self.__data[key], self, key)
        del self.__data[key]



import unittest

from zope.configuration.xmlconfig import XMLConfig
from zope.interface import implementer
from zope.publisher.browser import TestRequest
from zope.publisher.interfaces.browser import IBrowserPublisher

from zope.container.interfaces import IItemContainer
from zope.container.interfaces import ISimpleReadContainer
from zope.container.traversal import ItemTraverser
from zope.container.testing import ContainerPlacelessSetup


class ZCMLDependencies(ContainerPlacelessSetup, unittest.TestCase):

    def test_zcml_can_load(self):
        # this is just an example.  It is supposed to show that the
        # configure.zcml file has loaded successfully.

        import zope.container
        XMLConfig('configure.zcml', zope.container)()

        request = TestRequest()

        @implementer(IItemContainer)
        class SampleItemContainer(object):
            pass

        sampleitemcontainer = SampleItemContainer()
        res = zope.component.getMultiAdapter(
            (sampleitemcontainer, request), IBrowserPublisher)
        self.assertTrue(isinstance(res, ItemTraverser))
        self.assertTrue(res.context is sampleitemcontainer)

        @implementer(ISimpleReadContainer)
        class SampleSimpleReadContainer(object):
            pass

        samplesimplereadcontainer = SampleSimpleReadContainer()
        res = zope.component.getMultiAdapter(
            (samplesimplereadcontainer, request), IBrowserPublisher)
        self.assertTrue(isinstance(res, ItemTraverser))
        self.assertTrue(res.context is samplesimplereadcontainer)

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(ZCMLDependencies))
    return suite


if __name__ == '__main__':
    unittest.main()



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""This module provides a sample btree container implementation.
"""
__docformat__ = 'restructuredtext'

from persistent import Persistent
from BTrees.OOBTree import OOBTree
from BTrees.Length import Length

from zope.container.interfaces import IBTreeContainer
from zope.container.contained import Contained, setitem, uncontained
from zope.interface import implementer


class Lazy(object):
    #Lazy Attributes.


    def __init__(self, func, name=None):
        if name is None:
            name = func.__name__
        self.data = (func, name)

    def __get__(self, inst, class_):
        if inst is None:
            return self

        func, name = self.data
        value = func(inst)
        inst.__dict__[name] = value

        return value


@implementer(IBTreeContainer)
class BTreeContainer(Contained, Persistent):
    """OOBTree-based container"""

    def __init__(self):
        # We keep the previous attribute to store the data
        # for backward compatibility
        self._SampleContainer__data = self._newContainerData()
        self.__len = Length()

    def _newContainerData(self):
        """Construct an item-data container

        Subclasses should override this if they want different data.

        The value returned is a mapping object that also has get,
        has_key, keys, items, and values methods.
        The default implementation uses an OOBTree.
        """
        return OOBTree()

    def __contains__(self, key):
        """See interface IReadContainer
        """
        return key in self._SampleContainer__data

    @Lazy
    def _BTreeContainer__len(self):
        l = Length()
        ol = len(self._SampleContainer__data)
        if ol > 0:
            l.change(ol)
        self._p_changed = True
        return l

    def __len__(self):
        return self.__len()

    def _setitemf(self, key, value):
        # make sure our lazy property gets set
        l = self.__len
        self._SampleContainer__data[key] = value
        l.change(1)

    def __iter__(self):
        return iter(self._SampleContainer__data)

    def __getitem__(self, key):
        '''See interface `IReadContainer`'''
        return self._SampleContainer__data[key]

    def get(self, key, default=None):
        '''See interface `IReadContainer`'''
        return self._SampleContainer__data.get(key, default)

    def __setitem__(self, key, value):
        setitem(self, self._setitemf, key, value)

    def __delitem__(self, key):
        # make sure our lazy property gets set
        l = self.__len
        item = self._SampleContainer__data[key]
        del self._SampleContainer__data[key]
        l.change(-1)
        uncontained(item, self, key)

    has_key = __contains__

    def items(self, key=None):
        return self._SampleContainer__data.items(key)

    def keys(self, key=None):
        return self._SampleContainer__data.keys(key)

    def values(self, key=None):
        return self._SampleContainer__data.values(key)
